# Gridlock - a collection of grid-based strategy games
# Copyright (c) 2009 Brian Nenninger (brian@dozingcatsoftware.com)
# Gridlock comes with ABSOLUTELY NO WARRANTY, and is released under version 3 of the GNU GPL.

import GridUtils
from Game import GameState

# This class is subclassed by Dominion
class AtaxxGameState(GameState):
    def is_game_over(self):
        # game over when there are no empty cells, or none of either player
        return any(x not in self.grid for x in (0,1,2))

    def grid_with_flipped_neighbors(self, ec, pnum):
        nextgrid = self.grid.copy()
        nextgrid[ec] = pnum
        for nc,nv in self.enumerate_neighbors(ec):
            if nv>0 and nv!=pnum:
                nextgrid[nc] = pnum
        return nextgrid
                
    def successors(self):
        found = False
        pnum = self.current_player
        growcoords = set()
        for pc in self.enumerate_coords_with_value(pnum):
            # check empty 1-neighbors to grow
            for ec in self.enumerate_neighbor_coords_with_value(pc, 0):
                # grow to ec and flip its neighbors if we haven't seen ec before
                if ec not in growcoords:
                    nextgrid = self.grid_with_flipped_neighbors(ec, pnum)
                    growcoords.add(ec)
                    found = True
                    yield ( (ec,), self.next_state_with_grid(nextgrid) )
            # check empty 2-neighbors to jump, must be exactly 2 away
            for ec in self.enumerate_neighbor_coords_with_value(pc, 0, 2):
                if GridUtils.max_coord_diff(pc,ec)==2:
                    # jump to ec and flip neighbors
                    nextgrid = self.grid_with_flipped_neighbors(ec, pnum)
                    nextgrid[pc] = 0
                    found = True
                    yield ( (pc,ec), self.next_state_with_grid(nextgrid) )
        # if no moves found, pass
        if not found:
            yield ( (), self.next_state_with_grid(self.grid.copy()) )

    def animations_for_move(self, move, newstate):
        anim_group = GameState.animations_for_move(self, move, newstate)
        # if piece is being removed, convert it to a move to the jump destination
        import PGUtils, PGPieceAnimation
        remove_anim = PGUtils.first_item(a for a in anim_group.animations if a.segments[0].action==PGPieceAnimation.REMOVE)
        if remove_anim:
            new_anim = PGUtils.first_item(a for a in anim_group.animations if a.segments[0].action==PGPieceAnimation.NEW)
            if new_anim:
                # remove "remove" and "new" animations, replace with move
                animations = [a for a in anim_group.animations if a not in (new_anim, remove_anim)]
                animations.append(PGPieceAnimation.move_animation(remove_anim.position, new_anim.segments[0].from_position))
                anim_group = PGPieceAnimation.AnimationGroup(animations)
        return anim_group
        

                    
def test_ataxx():
    a = AtaxxGame((6,6))
    a.grid[0,0] = 1
    a.grid[5,5] = 1
    a.grid[0,5] = 2
    a.grid[5,0] = 2
    return a